from django.conf.urls import url

import views

urlpatterns = [
    url(r'^$', views.PoolList.as_view(), name='pool-list'),
    url(r'^pool/(?P<pool>[\w\\.-]+)/rebalance/$', views.PoolRebalance.as_view(), name='pool-rebalance'),
    url(r'^pool/(?P<pool>[\w\\.-]+)/$', views.PoolInfo.as_view(), name='pool-info'),
    url(r'^pool/(?P<pool>[\w\\.-]+)/metrics/$', views.PoolMetrics.as_view(), name='pool-metrics'),
    url(r'^node/(?P<address>.+)/remove/$', views.NodeRemove.as_view(), name='node-remove'),
    url(r'^node/add/$', views.NodeAdd.as_view(), name='node-add'),
    url(r'^deploys/$', views.ListDeploy.as_view(), name='list-deploys'),
    url(r'^deploys/(?P<deploy>[\s\w@\.-]+)/$', views.DeployInfo.as_view(), name='deploy-info'),
    url(r'^healing/$', views.ListHealing.as_view(), name='list-healing'),
    url(r'^templates.json$', views.TemplateListJson.as_view(), name='template-list-json'),
    url(r'^(?P<address>.+)/containers/$', views.NodeInfoJson.as_view(), name='node-info-json'),
    url(r'^(?P<address>.+)/$', views.NodeInfo.as_view(), name='node-info'),
]



from django import template

register = template.Library()


@register.filter
def divide(value, arg):
    try:
        return "%g" % (float(value) / int(arg))
    except (ValueError, ZeroDivisionError):
        return None



import json

from mock import patch, Mock

from django.test import TestCase
from django.test.client import RequestFactory
from django.core.urlresolvers import reverse

from tsuru_dashboard import settings
from tsuru_dashboard.auth.views import KeysRemove


class KeysRemoveViewTest(TestCase):
    @patch("requests.delete")
    @patch("requests.get")
    def test_view(self, get, delete):
        get.return_value = Mock(status_code=200)
        request = RequestFactory().get("/")
        request.session = {"tsuru_token": "admin"}
        key = "rsa"

        response = KeysRemove.as_view()(request, key=key)
        payload = {'name': key}

        self.assertEqual(302, response.status_code)
        self.assertEqual(reverse("list-keys"), response.items()[2][1])
        url = '{}/users/keys'.format(settings.TSURU_HOST)
        delete.assert_called_with(url, data=json.dumps(payload), headers={'authorization': 'admin'})



from time import mktime
from datetime import datetime, timedelta
import logging

import requests

from tsuru_dashboard.metrics.backends import base


logger = logging.getLogger(__name__)


class NoDataException(Exception):
    pass


class Prometheus(object):
    def __init__(self, url, date_range=None):
        self.url = url
        self.date_range = date_range

    @property
    def delta(self):
        if not self.date_range:
            return {"hours": 1}

        if "h" in self.date_range:
            return {"hours": int(self.date_range.replace("h", ""))}

        if "d" in self.date_range:
            return {"days": int(self.date_range.replace("d", ""))}

        if "w" in self.date_range:
            return {"days": int(self.date_range.replace("w", ""))*7}

    @property
    def start(self):
        return self.end - timedelta(**self.delta)

    @property
    def end(self):
        return datetime.now()

    @property
    def resolution(self):
        return (self.end - self.start).total_seconds() / 250

    def default_processor(self, results):
        def toMs(x):
            if len(x) == 2:
                return [x[0]*1000, x[1]]
            return x
        return list(map(toMs, results))

    def get_metrics(self, query, processor=None):
        url = self.url
        url += "/api/v1/query_range"
        params = {
            'query': query,
            'start': mktime(self.start.timetuple()),
            'end': mktime(self.end.timetuple()),
            'step': self.resolution,
        }
        result = requests.get(url, params=params)
        response_data = result.json()
        logger.debug("Prometheus request: {}\nPrometheus response {}: {} - {}".format(
                     result.url, result.status_code, result.headers, response_data))

        if processor is None:
            result_data = response_data.get('data', {}).get('result', [])
            if len(result_data) == 0:
                raise NoDataException()
            response_data = result_data[0].get('values', [])
            processor = self.default_processor

        return processor(response_data)


class AppBackend(Prometheus):

    def __init__(self, app, url, process_name=None, date_range=None):
        query = 'container_label_app_name="%s"' % app["name"]
        if process_name is not None:
            query += ',container_label_app_process="%s"' % process_name
        self.app_name = app["name"]
        self.process_name = process_name
        self.app_metric_templates = [
            '{container_metric}{{container_name=~"{app_name}-{process_name}.*",{extra_query}}}',
            '{container_metric}{{container_name="POD",pod_name=~"{app_name}-{process_name}.*",{extra_query}}}',
            '{container_metric}{{container_label_app_name="{app_name}",container_label_app_process{process_name_op}"{process_name}",{extra_query}}}',
        ]
        return super(AppBackend, self).__init__(
            url=url,
            date_range=date_range,
        )

    def mem_max(self, interval=None):
        data = {"min": 0, "max": 1024}
        query = self.make_query(
            "container_memory_usage_bytes",
            "avg({})/1024/1024",
        )
        data["avg"] = self.get_metrics(query)

        query = self.make_query(
            "container_memory_usage_bytes",
            "max({})/1024/1024",
        )
        data["max"] = self.get_metrics(query)

        query = self.make_query(
            "container_memory_usage_bytes",
            "min({})/1024/1024",
        )
        data["min"] = self.get_metrics(query)
        return {"data": data}

    def cpu_max(self, interval=None):
        data = {"min": 0, "max": 100}
        query = self.make_query(
            "container_cpu_usage_seconds_total",
            "avg(sum(rate({}[2m])) by (name)) * 100",
        )
        data["avg"] = self.get_metrics(query)

        query = self.make_query(
            "container_cpu_usage_seconds_total",
            "max(sum(rate({}[2m])) by (name)) * 100",
        )
        data["max"] = self.get_metrics(query)

        query = self.make_query(
            "container_cpu_usage_seconds_total",
            "min(sum(rate({}[2m])) by (name)) * 100",
        )
        data["min"] = self.get_metrics(query)
        return {"data": data}

    def units(self, interval=None):
        data = {"min": 0, "max": 100}
        query = self.make_query(
            "container_memory_usage_bytes",
            "max(count(rate({}[2m])) by (slave))",
        )
        data["units"] = self.get_metrics(query)
        return {"data": data}

    def swap(self, interval=None):
        data = {"min": 0, "max": 1024}
        query = self.make_query(
            "container_memory_swap",
            "avg({})/1024/1024",
        )
        data["avg"] = self.get_metrics(query)

        query = self.make_query(
            "container_memory_swap",
            "max({})/1024/1024",
        )
        data["max"] = self.get_metrics(query)

        query = self.make_query(
            "container_memory_swap",
            "min({})/1024/1024",
        )
        data["min"] = self.get_metrics(query)
        return {"data": data}

    def netrx(self, interval=None):
        data = {"min": 0, "max": 100}
        query = self.make_query(
            "container_network_receive_bytes_total",
            "sum(rate({}[2m]))/1024",
        )
        data["netrx"] = self.get_metrics(query)
        return {"data": data}

    def nettx(self, interval=None):
        data = {"min": 0, "max": 100}
        query = self.make_query(
            "container_network_transmit_bytes_total",
            "sum(rate({}[2m]))/1024",
        )
        data["nettx"] = self.get_metrics(query)
        return {"data": data}

    def connections(self, interval=None):
        data = {"min": 0, "max": 100}
        query = self.make_query(
            "container_connections",
            "sum({}) by (destination)",
            query='state="ESTABLISHED",protocol="tcp"',
        )
        data['data'] = self.get_metrics(query, processor=self.connections_processor)
        return data

    def connections_processor(self, result):
        results = result['data']['result']
        data = {}
        for r in results:
            data[base.set_destination_hostname(r['metric']['destination'])] = self.default_processor(r['values'])
        return data

    def make_query(self, metric, agg, query="", container_name_label="container_name"):
        args = {
            "container_metric": metric,
            "app_name": self.app_name,
            "process_name": self.process_name or "",
            "container_name_label": container_name_label,
            "extra_query": query,
        }
        args["process_name_op"] = "=" if self.process_name else "!="
        result = []
        for tpl in self.app_metric_templates:
            result.append(agg.format(tpl.format(**args)))
        return ' or '.join(result)



from tsuru_dashboard import engine


class ResourcesTab(engine.Tab):
    name = 'resources'
    url_name = 'app-resources'


class InfoTab(engine.Tab):
    name = 'info'
    url_name = 'app-info'


class DeploysTab(engine.Tab):
    name = 'deploys'
    url_name = 'app-deploys'


class EventsTab(engine.Tab):
    name = 'events'
    url_name = 'app-events'


class LogTab(engine.Tab):
    name = 'log'
    url_name = 'app-log'


class SettingsTab(engine.Tab):
    name = 'settings'
    url_name = 'app-settings'


class App(engine.App):
    name = 'app'

    def __init__(self):
        self.tabs = [ResourcesTab, InfoTab, DeploysTab, EventsTab, LogTab, SettingsTab]


engine.register(App)



from django.conf.urls import url, include
from django.contrib.staticfiles import views


urlpatterns = [
    url(r'^', include('tsuru_dashboard.urls')),
    url(r'^static/(?P<path>.*)$', views.serve),
]



from django.test import TestCase
from django.urls import reverse, resolve


class TestUrls(TestCase):

    def test_list_service_url(self):
        resolver = resolve('/services/')
        self.assertEqual(resolver.view_name, 'service-list')

    def test_service_detail_url(self):
        tests = [
            ["myservice", "myinstance"],
            ["my-service", "myinstance"],
            ["my_service", "myinstance"],
            ["my::service", "myinstance"],
        ]

        for test in tests:
            url = reverse('service-detail', args=test)
            self.assertEqual(url, '/services/'+test[0]+'/instances/'+test[1]+'/')
            resolver = resolve(url)
            self.assertEqual(resolver.view_name, 'service-detail')

    def test_service_add_url(self):
        tests = [
            ["myservice"],
            ["my-service"],
            ["my_service"],
            ["my::service"],
        ]

        for test in tests:
            url = reverse('service-add', args=test)
            self.assertEqual(url, '/services/'+test[0]+'/add/')
            resolver = resolve(url)
            self.assertEqual(resolver.view_name, 'service-add')

    def test_service_remove_url(self):
        tests = [
            ["myservice", "myinstance"],
            ["my-service", "myinstance"],
            ["my_service", "myinstance"],
            ["my::service", "myinstance"],
        ]

        for test in tests:
            url = reverse('service-remove', args=test)
            self.assertEqual(url, '/services/'+test[0]+'/instances/'+test[1]+'/remove/')
            resolver = resolve(url)
            self.assertEqual(resolver.view_name, 'service-remove')

    def test_service_bind_url(self):
        tests = [
            ["myservice", "myinstance"],
            ["my-service", "myinstance"],
            ["my_service", "myinstance"],
            ["my::service", "myinstance"],
        ]

        for test in tests:
            url = reverse('bind', args=test)
            self.assertEqual(url, '/services/'+test[0]+'/instances/'+test[1]+'/bind/')
            resolver = resolve(url)
            self.assertEqual(resolver.view_name, 'bind')

    def test_service_unbind_url(self):
        tests = [
            ["myservice", "myinstance", "myapp"],
            ["my-service", "myinstance", "myapp"],
            ["my_service", "myinstance", "myapp"],
            ["my::service", "myinstance", "myapp"],
        ]

        for test in tests:
            url = reverse('unbind', args=test)
            self.assertEqual(url, '/services/'+test[0]+'/instances/'+test[1]+'/'+test[2]+'/unbind/')
            resolver = resolve(url)
            self.assertEqual(resolver.view_name, 'unbind')



from mock import patch, Mock

import httpretty
import json

from django.test import TestCase
from django.test.client import RequestFactory

from tsuru_dashboard import settings
from tsuru_dashboard.apps.views import EventList


class EventListViewTest(TestCase):
    @patch('requests.get')
    @patch("tsuru_dashboard.auth.views.token_is_valid")
    def setUp(self, token_is_valid, get):
        token_is_valid.return_value = True
        self.request = RequestFactory().get("/?page=2")
        self.request.session = {"tsuru_token": "admin"}
        self.response = EventList.as_view()(self.request, app_name="appname")

    def test_context_should_contain_app(self):
        self.assertIn('app', self.response.context_data.keys())

    @patch('requests.get')
    @patch("tsuru_dashboard.auth.views.token_is_valid")
    def test_template(self, token_is_valid, get):
        token_is_valid.return_value = True
        get.return_value = Mock(status_code=200)
        self.assertIn("apps/events.html", self.response.template_name)
        self.assertIn('events', self.response.context_data)

    @patch('requests.get')
    @patch("tsuru_dashboard.auth.views.token_is_valid")
    def test_event_list(self, token_is_valid, get):
        token_is_valid.return_value = True
        self.request = RequestFactory().get("/")
        self.request.session = {"tsuru_token": "admin"}

        view = EventList
        view.get_app = Mock()
        self.response = view.as_view()(self.request, app_name="appname")

        url = "{}/events?skip=0&limit=20&target.type=app&target.value=appname"
        url = url.format(settings.TSURU_HOST)
        headers = {'authorization': 'admin'}
        get.assert_called_with(url, headers=headers)

    @patch('requests.get')
    @patch("tsuru_dashboard.auth.views.token_is_valid")
    def test_empty_list(self, token_is_valid, get):
        response_mock = Mock(status_code=200)
        response_mock.json.return_value = None
        get.return_value = response_mock
        token_is_valid.return_value = True

        request = RequestFactory().get("/")
        request.session = {"tsuru_token": "admin"}

        view = EventList
        view.get_app = Mock()
        view.as_view()(request, app_name="appname")

        url = '{}/events?skip=0&limit=20&target.type=app&target.value=appname'
        url = url.format(settings.TSURU_HOST)
        headers = {'authorization': 'admin'}
        get.assert_called_with(url, headers=headers)

    @httpretty.activate
    @patch("tsuru_dashboard.auth.views.token_is_valid")
    def test_pagination_page1(self, token_is_valid):
        token_is_valid.return_value = True

        url = '{}/events'.format(settings.TSURU_HOST)
        body = json.dumps([{
            "EndTime": u'2016-08-05T16:35:28.946-03:00',
            "StartTime": u'2016-08-05T16:35:28.835-03:00'
        }] * 1000)
        httpretty.register_uri(httpretty.GET, url, body=body, status=200)

        self.request = RequestFactory().get("/")
        self.request.session = {"tsuru_token": "admin"}

        response = EventList.as_view()(self.request, app_name="bla")

        self.assertEqual("page=2", response.context_data["next"])
        self.assertEqual(None, response.context_data.get("previous"))

    @httpretty.activate
    @patch("tsuru_dashboard.auth.views.token_is_valid")
    def test_pagination_page2(self, token_is_valid):
        token_is_valid.return_value = True

        url = '{}/events'.format(settings.TSURU_HOST)
        body = json.dumps([{
            "EndTime": u'2016-08-05T16:35:28.946-03:00',
            "StartTime": u'2016-08-05T16:35:28.835-03:00'
        }] * 1000)
        httpretty.register_uri(httpretty.GET, url, body=body, status=200)

        response = EventList.as_view()(self.request, app_name="bla")

        self.assertEqual("page=3", response.context_data["next"])
        self.assertEqual("", response.context_data["previous"])

    @httpretty.activate
    @patch("tsuru_dashboard.auth.views.token_is_valid")
    def test_pagination_page3(self, token_is_valid):
        token_is_valid.return_value = True

        url = '{}/events'.format(settings.TSURU_HOST)
        body = json.dumps([{
            "EndTime": u'2016-08-05T16:35:28.946-03:00',
            "StartTime": u'2016-08-05T16:35:28.835-03:00'
        }] * 1000)
        httpretty.register_uri(httpretty.GET, url, body=body, status=200)

        self.request = RequestFactory().get("/?page=3")
        self.request.session = {"tsuru_token": "admin"}

        response = EventList.as_view()(self.request, app_name="bla")

        self.assertEqual("page=4", response.context_data["next"])
        self.assertEqual("page=2", response.context_data["previous"])



from tsuru_dashboard.admin.views import NodeInfo
from mock import patch
from django.test import TestCase
from django.test.client import RequestFactory


class NodeInfoViewTest(TestCase):
    @patch("tsuru_dashboard.auth.views.token_is_valid")
    def test_should_use_node_info_template(self, token_is_valid):
        token_is_valid.return_value = True

        factory = RequestFactory()
        request = factory.get('/')
        request.session = {'tsuru_token': 'tokentest'}

        response = NodeInfo.as_view()(request, address="http://127.0.0.1")
        self.assertIn("admin/node_info.html", response.template_name)



from django.test import TestCase
from tsuru_dashboard.metrics.backends.prometheus import Prometheus, AppBackend
from freezegun import freeze_time

from mock import patch, Mock
from datetime import datetime, timedelta
from time import mktime


class AppBackendTest(TestCase):
    def setUp(self):
        self.backend = AppBackend({"name": "myapp"}, "http://url.com", process_name="myprocess")

    @freeze_time("2012-04-01 16:32:15", tz_offset=0)
    @patch("requests.get")
    def test_mem_max(self, get_mock):
        response_mock = Mock(status_code=200)
        response_mock.json.return_value = {
            "data": {
                "result": [{
                    "values": [],
                }],
            },
        }
        get_mock.return_value = response_mock

        self.backend.mem_max()
        start = datetime.now() - timedelta(hours=1)
        end = datetime.now()
        expected_url = self.backend.url + "/api/v1/query_range"
        expected_params = {
            'query': 'min(container_memory_usage_bytes{container_name=~"myapp-myprocess.*",})/1024/1024 or min(container_memory_usage_bytes{container_name="POD",pod_name=~"myapp-myprocess.*",})/1024/1024 or min(container_memory_usage_bytes{container_label_app_name="myapp",container_label_app_process="myprocess",})/1024/1024',
            'start': mktime(start.timetuple()),
            'end': mktime(end.timetuple()),
            'step': self.backend.resolution,
        }
        get_mock.assert_called_with(expected_url, params=expected_params)


class PrometheusTest(TestCase):
    def setUp(self):
        self.backend = Prometheus("http://url.com")

    @freeze_time("2012-04-01 16:32:15", tz_offset=0)
    def test_default_date_range(self):
        backend = Prometheus(
            "http://url.com",
        )
        self.assertEqual(backend.start, datetime.now() - timedelta(hours=1))
        self.assertEqual(backend.end, datetime.now())

    @freeze_time("2012-04-01 16:32:15", tz_offset=0)
    def test_hour_date_range(self):
        backend = Prometheus(
            "http://url.com",
            date_range="1h",
        )
        self.assertEqual(backend.start, datetime.now() - timedelta(hours=1))
        self.assertEqual(backend.end, datetime.now())

        backend = Prometheus(
            "http://url.com",
            date_range="3h",
        )
        self.assertEqual(backend.start, datetime.now() - timedelta(hours=3))
        self.assertEqual(backend.end, datetime.now())

    @freeze_time("2012-04-01 16:32:15", tz_offset=0)
    def test_date_range_by_days(self):
        backend = Prometheus(
            "http://url.com",
            date_range="1d",
        )
        self.assertEqual(backend.start, datetime.now() - timedelta(days=1))
        self.assertEqual(backend.end, datetime.now())

        backend = Prometheus(
            "http://url.com",
            date_range="3d",
        )
        self.assertEqual(backend.start, datetime.now() - timedelta(days=3))
        self.assertEqual(backend.end, datetime.now())

    @freeze_time("2012-04-01 16:32:15", tz_offset=0)
    def test_date_range_by_week(self):
        backend = Prometheus(
            "http://url.com",
            date_range="1w",
        )
        self.assertEqual(backend.start, datetime.now() - timedelta(days=7))
        self.assertEqual(backend.end, datetime.now())

        backend = Prometheus(
            "http://url.com",
            date_range="2w",
        )
        self.assertEqual(backend.start, datetime.now() - timedelta(days=14))
        self.assertEqual(backend.end, datetime.now())

    @freeze_time("2012-04-01 16:32:15", tz_offset=0)
    def test_resolution(self):
        backend = Prometheus(
            "http://url.com",
            date_range="1w",
        )
        resolution = (backend.end - backend.start).total_seconds() / 250
        self.assertEqual(backend.resolution, resolution)



from mock import patch, Mock

from django.test import TestCase
from django.test.client import RequestFactory

from tsuru_dashboard import settings
from tsuru_dashboard.admin.views import PoolList

import json
import httpretty


class PoolListViewTest(TestCase):
    def setUp(self):
        self.maxDiff = None
        self.request = RequestFactory().get("/")
        self.request.session = {"tsuru_token": "admin"}

    @patch("requests.get")
    @patch("tsuru_dashboard.auth.views.token_is_valid")
    def test_should_use_list_template(self, token_is_valid, get):
        token_is_valid.return_value = True

        response_mock = Mock()
        response_mock.json.return_value = {}
        get.return_value = response_mock

        response = PoolList.as_view()(self.request)

        self.assertIn("admin/pool_list.html", response.template_name)
        self.assertEqual([], response.context_data["pools"])
        url = "{}/docker/node".format(settings.TSURU_HOST)
        get.assert_called_with(url, headers={"authorization": "admin"})

    @httpretty.activate
    @patch("tsuru_dashboard.auth.views.token_is_valid")
    def test_should_pass_addresses_to_the_template(self, token_is_valid):
        token_is_valid.return_value = True

        url = "{}/docker/node".format(settings.TSURU_HOST)
        data = {
            "nodes": [
                {"Address": "http://128.0.0.1:4243",
                 "Status": "ready",
                 "Pool": "theonepool"},
                {"Address": "http://127.0.0.1:2375",
                 "Status": "ready",
                 "Pool": "theonepool"},
                {"Address": "http://myserver.com:2375",
                 "Status": "ready",
                 "Pool": "theonepool"},
                {"Address": "https://myserver.com:2376",
                 "Status": "ready",
                 "Pool": "dev\\dev.example.com"},
                {"Address": "https://myserver2.com:2376",
                 "Status": "ready",
                 "Pool": "dev\\dev.example.com"},
                {"Address": "kubernetes nodes",
                 "Status": "ERROR: unable to connect to cluster c1: http://c1.somewhere: connection refused",
                 "Pool": "kube"},
            ],
        }
        body = json.dumps(data)
        httpretty.register_uri(httpretty.GET, url, body=body)
        response = PoolList.as_view()(self.request)

        expected = {
            "kube": json.dumps([
                {"address": "kubernetes nodes",
                 "status": "ERROR: unable to connect to cluster c1: http://c1.somewhere: connection refused"},
            ]),
            "theonepool": json.dumps([
                {"address": "http://128.0.0.1:4243",
                 "status": "ready"},
                {"address": "http://127.0.0.1:2375",
                 "status": "ready"},
                {"address": "http://myserver.com:2375",
                 "status": "ready"},
            ]),
            "dev\\dev.example.com": json.dumps([
                {"address": "https://myserver.com:2376",
                 "status": "ready"},
                {"address": "https://myserver2.com:2376",
                 "status": "ready"},
            ]),
        }
        self.assertEqual(sorted(expected.items()),
                         response.context_data["pools"])

    @httpretty.activate
    @patch("tsuru_dashboard.auth.views.token_is_valid")
    def test_should_map_nodes_without_pool(self, token_is_valid):
        token_is_valid.return_value = True

        url = "{}/docker/node".format(settings.TSURU_HOST)
        data = {
            "nodes": [
                {"Address": "http://128.0.0.1:4243",
                 "Status": "ready",
                 "Pool": "theonepool"},
                {"Address": "http://127.0.0.1:2375",
                 "Status": "ready",
                 "Pool": ""},
                {"Address": "http://myserver.com:2375",
                 "Status": "ready",
                 "Pool": ""},
            ],
        }
        body = json.dumps(data)
        httpretty.register_uri(httpretty.GET, url, body=body)
        response = PoolList.as_view()(self.request)

        expected = {
            "theonepool": json.dumps([
                {"address": "http://128.0.0.1:4243",
                 "status": "ready"},
            ]),
            "": json.dumps([
                {"address": "http://127.0.0.1:2375",
                 "status": "ready"},
                {"address": "http://myserver.com:2375",
                 "status": "ready"},
            ]),
        }
        self.assertEqual(sorted(expected.items()),
                         response.context_data["pools"])



from tsuru_dashboard.auth.views import LoginRequiredMixin, PermissionRequiredMixin
from django.views.generic import TemplateView, View
from django.http import JsonResponse
from tsuru_dashboard import settings


class ListComponentJson(LoginRequiredMixin, PermissionRequiredMixin, View):
    def get(self, *args, **kwargs):
        component_list = {"components": [x.strip() for x in settings.METRICS_COMPONENTS.split(",")]}
        return JsonResponse(component_list, safe=False)


class ListComponent(LoginRequiredMixin, PermissionRequiredMixin, TemplateView):
    template_name = "components/list.html"



from django.conf.urls import url, include

from tsuru_dashboard.dashboard.views import IndexView

urlpatterns = [
    url(r'^$', IndexView.as_view()),
    url(r'^admin/', include('tsuru_dashboard.admin.urls')),
    url(r'^auth/', include('tsuru_dashboard.auth.urls')),
    url(r'^apps/', include('tsuru_dashboard.apps.urls')),
    url(r'^services/', include('tsuru_dashboard.services.urls')),
    url(r'^teams/', include('tsuru_dashboard.teams.urls')),
    url(r'^healthcheck/', include('tsuru_dashboard.healthcheck.urls')),
    url(r'^dashboard/', include('tsuru_dashboard.dashboard.urls')),
    url(r'^components/', include('tsuru_dashboard.components.urls')),
    url(r'^events/', include('tsuru_dashboard.events.urls')),
    url(r'^metrics/', include('tsuru_dashboard.metrics.urls')),
]



from mock import patch

from django.test import TestCase
from django.test.client import RequestFactory
from django.core.urlresolvers import reverse

import httpretty
import json

from tsuru_dashboard import settings
from tsuru_dashboard.admin.views import NodeInfoJson


class NodeInfoJsonViewTest(TestCase):
    @patch("tsuru_dashboard.auth.views.token_is_valid")
    def setUp(self, token_is_valid):
        token_is_valid.return_value = True
        httpretty.enable()

        self.factory = RequestFactory()
        self.request = self.factory.get('/')
        self.request.session = {'tsuru_token': 'tokentest'}

        self.address = 'http://127.0.0.2:4243'

        url = "{}/docker/node/{}/containers".format(settings.TSURU_HOST, 'http://127.0.0.2:4243')
        self.containers = [
            {"id": "blabla", "type": "python", "AppName": "myapp", "hostaddr": self.address, "IP": self.address}
        ]
        httpretty.register_uri(
            httpretty.GET,
            url,
            body=json.dumps(self.containers),
            status=200
        )

        url = "{}/docker/node".format(settings.TSURU_HOST, self.address)
        self.nodes = {
            "nodes": [
                {
                    "Status": "ready",
                    "Metadata": {
                        "pool": "tsuru1",
                        "iaas": "ec2",
                        "LastSuccess": "2015-11-16T18:44:36-02:00",
                    },
                    "Address": "http://127.0.0.2:4243"
                },
                {
                    "Status": "ready",
                    "Metadata": {
                        "pool": "tsuru2",
                        "iaas": "ec2",
                        "LastSuccess": "2015-11-16T18:44:36-02:00",
                    },
                    "Address": "http://127.0.0.3:4243"
                },
            ]
        }
        httpretty.register_uri(
            httpretty.GET,
            url,
            body=json.dumps(self.nodes),
            status=200
        )
        self.response = NodeInfoJson.as_view()(self.request, address=self.address)
        self.responseContent = json.loads(self.response.content)

    def tearDown(self):
        httpretty.disable()
        httpretty.reset()

    def test_should_get_list_of_containers_from_tsuru(self):
        containers = self.containers
        containers[0]["DashboardURL"] = u"/apps/myapp/info/"
        self.assertListEqual(containers, self.responseContent["node"]["info"]["units"])

    def test_should_get_node_info_from_tsuru(self):
        expected = {
            "status": "ready",
            "metadata": {
                "pool": "tsuru1",
                "iaas": "ec2",
                "LastSuccess": "2015-11-16T18:44:36-02:00",
            },
            "pool": "tsuru1",
            "last_success": '2015-11-16T20:44:36Z',
            "units": [
                {"DashboardURL": "/apps/myapp/info/", "id": "blabla", "type": "python",
                 "AppName": "myapp", "hostaddr": "http://127.0.0.2:4243", "IP": "http://127.0.0.2:4243"},
            ],
            "units_stats": {"total": 1},
            "address": "http://127.0.0.2:4243"
        }
        self.assertDictEqual(expected, self.responseContent["node"]["info"])

    def test_get_request_run_url_should_not_return_404(self):
        response = self.client.get(reverse('node-info-json', args=[self.address.replace("http://", "")]))
        self.assertNotEqual(404, response.status_code)

    def test_should_return_remove_node_url(self):
        expected = u"/admin/node/http://127.0.0.2:4243/remove/"
        self.assertEqual(expected, self.responseContent["node"]["nodeRemovalURL"])

    @patch("tsuru_dashboard.auth.views.token_is_valid")
    def test_should_get_list_of_containers_empty_list(self, token_is_valid):
        httpretty.reset()
        token_is_valid.return_value = True

        url = "{}/docker/node/{}/containers".format(settings.TSURU_HOST, 'http://127.0.0.2:4243')
        httpretty.register_uri(
            httpretty.GET,
            url,
            status=204
        )

        url = "{}/docker/node".format(settings.TSURU_HOST, self.address)
        httpretty.register_uri(
            httpretty.GET,
            url,
            body=json.dumps(self.nodes),
            status=200
        )
        response = NodeInfoJson.as_view()(self.request, address=self.address)
        self.assertListEqual([], json.loads(response.content)["node"]["info"]["units"])

    @patch("tsuru_dashboard.auth.views.token_is_valid")
    def test_should_get_list_of_containers_on_error(self, token_is_valid):
        httpretty.reset()
        token_is_valid.return_value = True

        url = "{}/docker/node/{}/containers".format(settings.TSURU_HOST, 'http://127.0.0.2:4243')
        httpretty.register_uri(
            httpretty.GET,
            url,
            status=500
        )

        url = "{}/docker/node".format(settings.TSURU_HOST, self.address)
        httpretty.register_uri(
            httpretty.GET,
            url,
            body=json.dumps(self.nodes),
            status=200
        )
        response = NodeInfoJson.as_view()(self.request, address=self.address)
        self.assertListEqual([], json.loads(response.content)["node"]["info"]["units"])

    @patch("tsuru_dashboard.auth.views.token_is_valid")
    def test_should_get_empty_node(self, token_is_valid):
        httpretty.reset()
        token_is_valid.return_value = True

        url = "{}/docker/node/{}/containers".format(settings.TSURU_HOST, 'http://127.0.0.2:4243')
        httpretty.register_uri(
            httpretty.GET,
            url,
            body=json.dumps(self.containers),
            status=200
        )

        url = "{}/docker/node".format(settings.TSURU_HOST, self.address)
        httpretty.register_uri(
            httpretty.GET,
            url,
            status=204
        )
        response = NodeInfoJson.as_view()(self.request, address=self.address)
        self.assertFalse(json.loads(response.content)["node"]["info"])

    @patch("tsuru_dashboard.auth.views.token_is_valid")
    def test_should_use_container_ip(self, token_is_valid):
        token_is_valid.return_value = True

        node_data = {
            "machines": None,
            "nodes": [
                {"Address": "http://128.0.0.1:4243"},
            ],
        }
        node_url = "{}/docker/node".format(settings.TSURU_HOST)
        httpretty.register_uri(httpretty.GET, node_url, body=json.dumps(node_data), status=200)

        container_data = [
            {"IP": "128.0.0.1"},
        ]
        container_url = "{}/docker/node/http://128.0.0.1:4243/containers".format(settings.TSURU_HOST)
        httpretty.register_uri(httpretty.GET, container_url, body=json.dumps(container_data), status=200)

        factory = RequestFactory()
        request = factory.get('/')
        request.session = {'tsuru_token': 'tokentest'}

        response = NodeInfoJson.as_view()(self.request, address='http://128.0.0.1:4243')
        node_info = json.loads(response.content)["node"]["info"]
        self.assertEquals({"total": 1}, node_info["units_stats"])
        self.assertEquals(container_data, node_info["units"])

    @patch("tsuru_dashboard.auth.views.token_is_valid")
    def test_should_accept_older_ip_field(self, token_is_valid):
        token_is_valid.return_value = True

        node_data = {
            "machines": None,
            "nodes": [
                {"Address": "http://128.0.0.1:4243"},
            ],
        }
        node_url = "{}/docker/node".format(settings.TSURU_HOST)
        httpretty.register_uri(httpretty.GET, node_url, body=json.dumps(node_data), status=200)

        container_data = [
            {"ID": 1, "Ip": "128.0.0.1"},
            {"ID": 2, "Ip": "128.0.0.1"},
        ]
        container_url = "{}/docker/node/http://128.0.0.1:4243/containers".format(settings.TSURU_HOST)
        httpretty.register_uri(httpretty.GET, container_url, body=json.dumps(container_data), status=200)

        factory = RequestFactory()
        request = factory.get('/')
        request.session = {'tsuru_token': 'tokentest'}

        response = NodeInfoJson.as_view()(self.request, address='http://128.0.0.1:4243')
        node_info = json.loads(response.content)["node"]["info"]
        self.assertEquals({"total": 2}, node_info["units_stats"])
        self.assertEquals(container_data, node_info["units"])



import requests
import grequests
import urllib
import json

from django.views.generic import TemplateView
from django.http import HttpResponse, Http404, JsonResponse, StreamingHttpResponse, QueryDict
from django.core.urlresolvers import reverse
from django.contrib import messages

from pygments import highlight
from pygments.lexers import DiffLexer
from pygments.formatters import HtmlFormatter

from tsuru_dashboard import settings
from tsuru_dashboard.auth.views import LoginRequiredView
from tsuru_dashboard.admin.models import Node


class PoolList(LoginRequiredView, TemplateView):
    template_name = "admin/pool_list.html"

    def nodes_by_pool(self):
        url = "{}/docker/node".format(settings.TSURU_HOST)
        response = requests.get(url, headers=self.authorization)
        pools = {}

        if response.status_code != 204:
            data = response.json()
            nodes = data.get("nodes", [])

            for node in nodes:
                pool = Node(node).pool()
                nodes_by_pool = pools.get(pool, [])
                data = {"address": node["Address"], "status": node["Status"]}
                nodes_by_pool.append(data)
                pools[pool] = nodes_by_pool

            for pool in pools:
                pools[pool] = json.dumps(pools[pool])
        return sorted(pools.items())

    def get_context_data(self, **kwargs):
        context = super(PoolList, self).get_context_data(**kwargs)
        context.update({"pools": self.nodes_by_pool()})
        return context


class NodeInfo(LoginRequiredView, TemplateView):
    template_name = "admin/node_info.html"


class NodeInfoJson(LoginRequiredView):
    def get_containers(self, node_address):
        url = "{}/docker/node/{}/containers".format(settings.TSURU_HOST, node_address)
        return requests.get(url, headers=self.authorization)

    def get_node(self, address):
        url = "{}/docker/node".format(settings.TSURU_HOST)
        response = requests.get(url, headers=self.authorization)

        if response.status_code == 204:
            return None

        data = response.json()
        nodes = data.get("nodes", [])

        for node in nodes:
            if node["Address"] == address:
                return node

        return None

    def get(self, *args, **kwargs):
        address = kwargs["address"]
        node_dict = None
        node = self.get_node(address)
        if node:
            containers_rsp = self.get_containers(address)
            node = Node(node, containers_rsp)
            node_dict = node.to_dict()
            for container in node_dict["units"]:
                if "AppName" in container:
                    container["DashboardURL"] = reverse(
                        'app-info', kwargs={'app_name': container["AppName"]})
        return JsonResponse({
            "node": {
                "info": node_dict,
                "nodeRemovalURL": reverse('node-remove', kwargs={'address': address})
            }
        })


class ListDeploy(LoginRequiredView, TemplateView):
    template_name = "deploys/list_deploys.html"

    def get_context_data(self, **kwargs):
        context = super(ListDeploy, self).get_context_data(**kwargs)

        page = int(self.request.GET.get('page', '1'))

        skip = (page * 20) - 20
        limit = page * 20

        url = '{}/deploys?skip={}&limit={}'.format(settings.TSURU_HOST, skip, limit)

        response = requests.get(url, headers=self.authorization)

        if response.status_code == 204:
            deploys = []
        else:
            deploys = response.json()

        context['deploys'] = deploys

        if len(deploys) >= 20:
            context['next'] = page + 1

        if page > 0:
            context['previous'] = page - 1

        return context


class DeployInfo(LoginRequiredView, TemplateView):
    template_name = "deploys/deploy_details.html"

    def get_context_data(self, **kwargs):
        deploy_id = kwargs["deploy"]

        url = "{}/deploys/{}".format(settings.TSURU_HOST, deploy_id)
        response = requests.get(url, headers=self.authorization)

        if response.status_code > 399:
            raise Http404("Deploy does not exist")

        context = {"deploy": response.json()}

        diff = context["deploy"].get("Diff")
        if diff and diff != u'The deployment must have at least two commits for the diff.':
            format = HtmlFormatter()
            diff = highlight(diff, DiffLexer(), format)
        else:
            diff = None

        context["deploy"]["Diff"] = diff
        return context


class ListHealing(LoginRequiredView, TemplateView):
    template_name = "admin/list_healing.html"

    def get_context_data(self, **kwargs):
        context = super(ListHealing, self).get_context_data(**kwargs)
        url = '{}/docker/healing'.format(settings.TSURU_HOST)
        response = requests.get(url, headers=self.authorization)
        formatted_events = []
        if response.status_code == 200:
            events = response.json() or []
            formatted_events = []

            for event in events:
                event['FailingContainer']['ID'] = event['FailingContainer']['ID'][:12]
                event['CreatedContainer']['ID'] = event['CreatedContainer']['ID'][:12]
                event['App'] = event['FailingContainer']['AppName']
                formatted_events.append(event)

        context.update({"events": formatted_events})
        return context


class PoolInfo(LoginRequiredView, TemplateView):
    template_name = "admin/pool_info.html"

    def nodes_by_pool(self, pool):
        url = "{}/docker/node".format(settings.TSURU_HOST)
        response = requests.get(url, headers=self.authorization)

        if response.status_code == 204:
            return []

        data = response.json()
        node_list = data.get("nodes", [])
        nodes = map(lambda n: Node(n), node_list)
        nodes = filter(lambda n: n.pool() == pool, nodes)

        url = "{}/docker/node/{}/containers"

        rs = []
        for node in nodes:
            u = url.format(settings.TSURU_HOST, node.address())
            rs.append(grequests.get(u, headers=self.authorization))

        units = grequests.map(rs)
        nodes_with_units = []

        for i, node in enumerate(nodes):
            node.load_units(units[i])
            nodes_with_units.append(node.to_dict())

        return nodes_with_units

    def get_context_data(self, **kwargs):
        context = super(PoolInfo, self).get_context_data(**kwargs)
        context.update({"nodes": self.nodes_by_pool(kwargs["pool"])})
        return context


class PoolMetrics(LoginRequiredView, TemplateView):
    template_name = "admin/pool_metrics.html"

    def get_grafana_url(self):
        if not settings.GRAFANA_POOL_DASHBOARD:
            return

        datasource = settings.GRAFANA_DATASOURCE_FOR_POOL.get(
            self.kwargs['pool'],
            settings.GRAFANA_DEFAULT_DATASOURCE
        )

        args = {
            'var-datasource': datasource,
            'var-pool': self.kwargs['pool'],
            'theme': settings.GRAFANA_THEME,
            'kiosk': settings.GRAFANA_KIOSK,
        }

        return settings.GRAFANA_POOL_DASHBOARD + '?' + urllib.urlencode(args)

    def get_context_data(self, **kwargs):
        context = super(PoolMetrics, self).get_context_data(**kwargs)
        context["grafana_url"] = self.get_grafana_url()
        return context


class NodeRemove(LoginRequiredView):
    def delete(self, *args, **kwargs):
        address = self.kwargs['address']
        params = QueryDict(self.request.body)

        msg = u"The value for '{}' parameter should be 'true' or 'false'"

        destroy = params.get("destroy", "false")
        if destroy not in ["true", "false"]:
            return HttpResponse(msg.format("destroy"), status=400)

        rebalance = params.get("rebalance", "false")
        if rebalance not in ["true", "false"]:
            return HttpResponse(msg.format("rebalance"), status=400)

        no_rebalance = "false" if rebalance == "true" else "true"

        response = requests.delete(
            '{}/docker/node/{}?remove-iaas={}&no-rebalance={}'.format(
                settings.TSURU_HOST, address, destroy, no_rebalance
            ),
            headers=self.authorization
        )

        if response.status_code > 399:
            return HttpResponse(response.text, status=response.status_code)

        return HttpResponse('Node was successfully removed', status=200)


class TemplateListJson(LoginRequiredView):

    def get(self, *args, **kwargs):
        templates = self.client.templates.list()
        return JsonResponse(templates, safe=False)


class NodeAdd(LoginRequiredView):

    def post(self, *args, **kwargs):
        resp = self.client.nodes.create(**self.request.POST.dict())

        for line in resp.iter_lines():
            msg_err = json.loads(line).get('Error')
            if msg_err:
                messages.error(self.request, msg_err, fail_silently=True)
                return HttpResponse(msg_err, status=500)

        messages.success(self.request, u'Node was successfully created', fail_silently=True)
        return HttpResponse('Node was successfully created', status=200)


class PoolRebalance(LoginRequiredView):

    def post(self, *args, **kwargs):
        def sending_stream():
            gen = ("{}<br>".format(line["Message"]) for line in self.client.pools.rebalance(pool=kwargs["pool"]))
            return gen

        return StreamingHttpResponse(sending_stream())



import json
import requests

from tsuruclient import client

from tsuru_dashboard import settings

from django.template.response import TemplateResponse
from django.shortcuts import redirect
from django.views.generic.base import View, TemplateView
from django.views.generic.edit import FormView
from django.core.urlresolvers import reverse
from django.contrib import messages
from django.utils.decorators import method_decorator
from django.views.decorators.csrf import ensure_csrf_cookie

from .forms import (LoginForm, SignupForm, KeyForm, TokenRequestForm,
                    PasswordRecoveryForm, ChangePasswordForm)


def get_oauth_redirect_url(request):
    scheme = request.scheme
    host = request.META.get('HTTP_HOST', request.META.get('SERVER_NAME'))

    return "{}://{}/auth/callback/".format(scheme, host)


def token_is_valid(token):
    headers = {'authorization': token}
    url = "{0}/users/info".format(settings.TSURU_HOST)
    response = requests.get(url, headers=headers)
    return response.status_code == 200


def get_permissions(token):
    headers = {"authorization": token}
    permissions = {}

    url = "{0}/users/info".format(settings.TSURU_HOST)
    response = requests.get(url, headers=headers)
    user = response.json()
    permissions["admin"] = False
    permissions["healing"] = False
    for perm in user["Permissions"]:
        if perm["Name"] == "" and perm["ContextType"] == "global":
            permissions["admin"] = True
            for k in permissions:
                permissions[k] = True
            return permissions
        elif perm["Name"] == "healing.read":
            permissions["healing"] = True

    return permissions


class LoginRequiredMixin(object):

    @property
    def client(self):
        target = settings.TSURU_HOST
        token = self.request.session.get('tsuru_token').split(" ")[-1]
        return client.Client(target, token)

    @property
    def authorization(self):
        return {'authorization': self.request.session.get('tsuru_token')}

    @method_decorator(ensure_csrf_cookie)
    def dispatch(self, request, *args, **kwargs):
        token = request.session.get('tsuru_token')
        if not token or not token_is_valid(token):
            return redirect("%s?next=%s" % (reverse('login'), request.path))
        return super(LoginRequiredMixin, self).dispatch(
            request, *args, **kwargs)


class PermissionRequiredMixin(object):
    required_permission = "admin"

    def dispatch(self, request, *args, **kwargs):
        permission = request.session.get('permissions', {}).get(self.required_permission, False)
        if not permission:
            messages.error(self.request, u'Permission denied.', fail_silently=True)
            return redirect('/')
        else:
            return super(PermissionRequiredMixin, self).dispatch(
                request, *args, **kwargs)


class ChangePassword(LoginRequiredMixin, FormView):
    template_name = 'auth/change_password.html'
    form_class = ChangePasswordForm
    success_url = '/auth/change-password/'

    def form_valid(self, form):
        data = form.cleaned_data
        headers = {'authorization': self.request.session.get('tsuru_token')}
        url = "{0}/users/password".format(settings.TSURU_HOST)
        response = requests.put(url, data=data, headers=headers)
        if response.status_code < 399:
            messages.success(self.request, u'Password successfully updated!', fail_silently=True)
        else:
            messages.error(self.request, response.text, fail_silently=True)
        return super(ChangePassword, self).form_valid(form)


class LoginRequiredView(LoginRequiredMixin, View):
    pass


class TokenRequest(FormView):
    template_name = 'auth/token_request.html'
    form_class = TokenRequestForm
    success_url = '/auth/token-request/success/'

    def form_valid(self, form):
        form.send()
        return super(TokenRequest, self).form_valid(form)


class TokenRequestSuccess(TemplateView):
    template_name = 'auth/token_request_success.html'


class PasswordRecoverySuccess(TemplateView):
    template_name = 'auth/password_recovery_success.html'


class PasswordRecovery(FormView):
    template_name = 'auth/password_recovery.html'
    form_class = PasswordRecoveryForm
    success_url = '/auth/password-recovery/success/'

    def form_valid(self, form):
        form.send()
        return super(PasswordRecovery, self).form_valid(form)


class Login(FormView):
    template_name = 'auth/login.html'
    form_class = LoginForm

    def get_context_data(self, *args, **kwargs):
        self.request.session["next_url"] = self.request.GET.get("next", "/apps")
        data = super(Login, self).get_context_data(*args, **kwargs)
        data["scheme_info"] = scheme = self.scheme_info()
        scheme_data = scheme.get('data', {})
        if scheme_data is None:
            return data
        authorize_url = scheme_data.get('authorizeUrl', '')
        callback_url = get_oauth_redirect_url(self.request)
        data["authorize_url"] = authorize_url.replace('__redirect_url__', callback_url)
        return data

    def get_success_url(self):
        return '/apps'

    def scheme_info(self):
        url = '{0}/auth/scheme'.format(settings.TSURU_HOST)
        response = requests.get(url)
        if response.status_code == 200:
            return response.json()
        return {}

    def form_valid(self, form):
        username = form.cleaned_data['username']
        data = {"password": form.cleaned_data['password']}
        url = '{0}/users/{1}/tokens'.format(settings.TSURU_HOST, username)
        response = requests.post(url, data=data)

        if response.status_code == 200:
            result = response.json()
            self.request.session['username'] = username
            self.request.session['tsuru_token'] = "type {0}".format(result['token'])
            self.request.session['permissions'] = get_permissions(result['token'])
            return super(Login, self).form_valid(form)

        form.add_error(None, response.content)
        return super(Login, self).form_invalid(form)


class Logout(View):

    def get(self, request):
        if 'tsuru_token' in request.session:
            del request.session['tsuru_token']
        return redirect('/')


class Signup(View):

    def get(self, request):
        return TemplateResponse(request, 'auth/signup.html',
                                context={'signup_form': SignupForm()})

    def post(self, request):
        form = SignupForm(request.POST)
        if not form.is_valid():
            return TemplateResponse(request,
                                    'auth/signup.html',
                                    context={'signup_form': form})
        post_data = {'email': form.data['email'],
                     'password': form.data['password']}
        response = requests.post(
            '%s/users' % settings.TSURU_HOST,
            data=json.dumps(post_data))
        if response.status_code == 201:
            message = 'User "{0}" successfully created!'.format(
                form.data['email']
            )
            return TemplateResponse(request, 'auth/signup.html',
                                    context={'signup_form': SignupForm(),
                                             'message': message})
        else:
            return TemplateResponse(request, 'auth/signup.html',
                                    context={'signup_form': form,
                                             'error': response.content},
                                    status=response.status_code)


class Callback(View):
    def get(self, request):
        code = request.GET.get("code")
        redirect_url = get_oauth_redirect_url(request)
        data = {
            "code": code,
            "redirectUrl": redirect_url,
        }
        url = '{}/auth/login'.format(settings.TSURU_HOST)
        response = requests.post(url, data=data)
        if response.status_code == 200:
            result = response.json()
            self.request.session['tsuru_token'] = "type {}".format(result['token'])
            self.request.session['permissions'] = get_permissions(result['token'])
            next_url = self.request.session["next_url"]
            return redirect(next_url)
        return redirect('/auth/login')


class KeysAdd(LoginRequiredMixin, FormView):
    form_class = KeyForm
    template_name = 'auth/key_add.html'
    success_url = '/auth/keys/add'

    def form_valid(self, form):
        url = '{}/users/keys'.format(settings.TSURU_HOST)
        response = requests.post(url, data=form.cleaned_data, headers=self.authorization)
        if response.status_code < 399:
            messages.success(self.request, "The key was successfully added", fail_silently=True)
        else:
            messages.error(self.request, response.text, fail_silently=True)
        return super(KeysAdd, self).form_valid(form)


class KeysList(LoginRequiredMixin, TemplateView):
    template_name = 'auth/key_list.html'

    def get_context_data(self, *args, **kwargs):
        context = super(KeysList, self).get_context_data(*args, **kwargs)
        url = '{}/users/keys'.format(settings.TSURU_HOST)
        response = requests.get(url, headers=self.authorization)
        keys = {}
        if response.status_code == 200:
            keys = response.json()

        context.update({"keys": keys})
        return context


class KeysRemove(LoginRequiredView):
    def get(self, request, *args, **kwargs):
        key_name = kwargs["key"]
        payload = {'name': key_name}

        url = '{}/users/keys'.format(settings.TSURU_HOST)
        requests.delete(url, data=json.dumps(payload), headers=self.authorization)
        return redirect(reverse('list-keys'))



from django.conf.urls import url

import views


urlpatterns = [
    url(r'^$', views.ListApp.as_view(), name='list-app'),
    url(r'^list.json$', views.ListAppJson.as_view(), name='list-app-json'),
    url(r'^create/$', views.CreateApp.as_view(), name='create-app'),
    url(r'^(?P<app_name>[\w-]+).json$',
        views.AppDetailJson.as_view(), name='detail-app-json'),
    url(r'^(?P<app_name>[\w-]+)/$',
        views.AppResources.as_view(), name='app-resources'),
    url(r'^(?P<app_name>[\w-]+)/info/$',
        views.AppInfo.as_view(), name='app-info'),
    url(r'^(?P<name>[\w-]+)/remove/$',
        views.RemoveApp.as_view(), name='remove_app'),
    url(r'^(?P<name>[\w-]+)/unlock/$', views.Unlock.as_view(), name='unlock-app'),
    url(r'^(?P<app_name>[\w-]+)/log/stream/$', views.LogStream.as_view(), name='app-log-stream'),
    url(r'^(?P<app_name>[\w-]+)/log/$', views.AppLog.as_view(),
        name='app-log'),
    url(r'^(?P<app_name>[\w-]+)/settings/$', views.Settings.as_view(),
        name='app-settings'),
    url(r'^(?P<app_name>[\w-]+)/deploys/$', views.ListDeploy.as_view(), name='app-deploys'),
    url(r'^(?P<app_name>[\w-]+)/deploys/(?P<deploy>[\s\w@\.-]+)/$',
        views.DeployInfo.as_view(), name='app-deploy'),
    url(r'^(?P<app_name>[\w-]+)/rollback/(?P<image>.+)/$',
        views.AppRollback.as_view(), name='app-rollback'),
    url(r'^(?P<app_name>[\w-]+)/events/$', views.EventList.as_view(), name='app-events'),
    url(r'^(?P<app_name>[\w-]+)/events/(?P<uuid>[\s\w@\.-]+)/$',
        views.EventInfo.as_view(), name='app-event'),
]



from base64 import encodestring
from cStringIO import StringIO
from django.urls.base import reverse
from django.test import TestCase
from django.test.client import RequestFactory
from mock import patch, Mock
from zipfile import ZipFile

from tsuru_dashboard import settings
from tsuru_dashboard.apps.views import ListDeploy


@patch("tsuru_dashboard.auth.views.token_is_valid", return_value=True)
class ListDeployViewTest(TestCase):
    def setUp(self):
        self.app_name = 'my-awesome-app'
        kwargs = {'app_name': self.app_name}
        self.app_deploy_path = reverse('app-deploys', kwargs=kwargs)
        self.session = {'tsuru_token': 'just-another_tsuru_token'}
        self.get_request = RequestFactory().get(self.app_deploy_path, kwargs=kwargs)
        self.get_request.session = self.session

    @patch('requests.get')
    def test_context_should_contain_app(self, get, token_is_valid):
        response = ListDeploy.as_view()(self.get_request, app_name=self.app_name)
        self.assertIn('app', response.context_data.keys())

    @patch('requests.get')
    def test_should_use_deploys_template(self, get, token_is_valid):
        response = ListDeploy.as_view()(self.get_request, app_name=self.app_name)
        self.assertEqual("apps/deploys.html", response.template_name)
        self.assertIn('deploys', response.context_data.keys())

    @patch('requests.get')
    def test_deploy_list(self, get, token_is_valid):
        token_is_valid.return_value = True
        self.request = RequestFactory().get("/")
        self.request.session = {"tsuru_token": "admin"}

        view = ListDeploy
        view.get_app = Mock()
        self.response = view.as_view()(self.request, app_name="appname")

        url = '{}/deploys?app=appname&skip=0&limit=20'.format(settings.TSURU_HOST)
        headers = {'authorization': 'admin'}
        get.assert_called_with(url, headers=headers)

    @patch('requests.get')
    def test_empty_list(self, get, token_is_valid):
        response_mock = Mock(status_code=200)
        response_mock.json.return_value = None
        get.return_value = response_mock
        token_is_valid.return_value = True

        request = RequestFactory().get("/")
        request.session = {"tsuru_token": "admin"}

        view = ListDeploy
        view.get_app = Mock()
        view.as_view()(request, app_name="appname")

        url = '{}/deploys?app=appname&skip=0&limit=20'.format(settings.TSURU_HOST)
        headers = {'authorization': 'admin'}
        get.assert_called_with(url, headers=headers)

    def test_app_deploy_when_receives_invalid_params_should_return_400_with_expected_error_message(self, token_is_valid):
        test_cases = [
            {'request': RequestFactory().post(self.app_deploy_path, {})},
            {'request': RequestFactory().post(self.app_deploy_path, {'filecontent': 'this string was not encoded with base64'})}
        ]
        expected = {
            'status_code': 400,
            'content': 'the "filecontent" parameter is missing or it is not well formatted',
        }
        for test_case in test_cases:
            request = test_case.get('request')
            request.session = self.session
            response = ListDeploy.as_view()(request, app_name=self.app_name)
            error_message = 'Failure at test case: %s' % test_case
            self.assertEqual(expected.get('status_code'), response.status_code, error_message)
            self.assertEqual(expected.get('content'), response.content, error_message)

    @patch('requests.post')
    def test_app_deploy_when_receives_valid_parameters_should_deploy_on_api(self, post, token_is_valid):
        def generate_test_zip():
            zip_file_handler = StringIO()
            with ZipFile(zip_file_handler, mode='w') as zip_object:
                zip_object.writestr('hello.txt', 'Hello world :)')
            zip_file_handler.seek(0)
            return zip_file_handler
        zip_file_handler = generate_test_zip()
        encoded_file_content = encodestring(zip_file_handler.getvalue())
        targz_file_handler = ListDeploy.create_targz_file_object_from_zip_file(zip_file_handler)
        expected_content = targz_file_handler.getvalue()
        targz_file_handler.close()
        zip_file_handler.close()
        post.return_value = Mock(iter_lines=Mock(return_value=iter(['OK'])))
        request = RequestFactory().post(self.app_deploy_path, {'filecontent': encoded_file_content})
        request.session = self.session
        response = ListDeploy.as_view()(request, app_name=self.app_name)
        url = '{}/apps/{}/deploy?origin={}'.format(settings.TSURU_HOST, self.app_name, 'drag-and-drop')
        self.assertEqual(response.getvalue(), 'OK<br>')
        expected_headers = {'authorization': self.session.get('tsuru_token')}
        expected_files = {'file': ('archive.tar.gz', expected_content)}
        post.assert_called_with(url, headers=expected_headers, files=expected_files, stream=True)



from django.test import TestCase
from tsuru_dashboard import settings
from tsuru_dashboard.metrics.backends import get_app, get_envs
from mock import patch, Mock


class BackendsTest(TestCase):
    @patch("requests.get")
    def test_get_app(self, get_mock):
        response_mock = Mock()
        response_mock.json.return_value = {}
        get_mock.return_value = response_mock

        app = get_app("app_name", "token")

        self.assertDictEqual(app, {})
        url = "{}/apps/app_name".format(settings.TSURU_HOST)
        headers = {"authorization": "token"}
        get_mock.assert_called_with(url, headers=headers)

    @patch("requests.get")
    def test_get_envs(self, get_mock):
        env_mock = [{"name": "VAR", "value": "value"}]
        response_mock = Mock()
        response_mock.json.return_value = env_mock
        get_mock.return_value = response_mock

        envs = get_envs("app_name", "token")

        self.assertDictEqual(envs, {"VAR": "value"})
        url = "{}/apps/app_name/env".format(settings.TSURU_HOST)
        headers = {"authorization": "token"}
        get_mock.assert_called_with(url, headers=headers)



from django.test import TestCase
from django.test.client import RequestFactory

from tsuru_dashboard.auth.views import get_oauth_redirect_url


class UtilsTest(TestCase):

    def test_get_oauth_redirect_url_when_request_is_insecure_should_be_scheme_equals_to_http(self):
        request = RequestFactory().get('/')
        request.META['HTTP_HOST'] = 'tsuru.local'

        redirect_oauth_url = get_oauth_redirect_url(request)

        self.assertEqual(redirect_oauth_url, 'http://tsuru.local/auth/callback/')

    def test_get_oauth_redirect_url_when_request_is_secure_should_be_scheme_equals_to_https(self):
        request = RequestFactory().get('/', secure=True)
        request.META['HTTP_HOST'] = 'tsuru.local'

        redirect_oauth_url = get_oauth_redirect_url(request)

        self.assertEqual(redirect_oauth_url, 'https://tsuru.local/auth/callback/')

    def test_get_oauth_redirect_url_when_connection_from_enduser_to_proxy_is_secure_should_be_scheme_equals_to_https(self):
        request = RequestFactory().get('/')
        request.META['HTTP_HOST'] = 'tsuru.local'
        request.META['HTTP_X_FORWARDED_PROTO'] = 'https'

        redirect_oauth_url = get_oauth_redirect_url(request)

        self.assertEqual(redirect_oauth_url, 'https://tsuru.local/auth/callback/')

    def test_get_oauth_redirect_url_when_header_host_is_undefined_should_use_fallback_server_name(self):
        request = RequestFactory().get('/')
        request.META['SERVER_NAME'] = 'localhost:8000'

        redirect_oauth_url = get_oauth_redirect_url(request)

        self.assertEqual(redirect_oauth_url, 'http://localhost:8000/auth/callback/')



from django import forms
from django.forms import widgets
from tsuru_dashboard import settings

import requests


class ChangePasswordForm(forms.Form):
    old = forms.CharField(widget=forms.PasswordInput())
    new = forms.CharField(widget=forms.PasswordInput())
    confirm = forms.CharField(widget=forms.PasswordInput())


class PasswordRecoveryForm(forms.Form):
    email = forms.EmailField()
    token = forms.CharField()

    def send(self):
        url = "{0}/users/{1}/password?token={2}".format(
            settings.TSURU_HOST,
            self.cleaned_data['email'],
            self.cleaned_data['token']
        )
        requests.post(url)


class TokenRequestForm(forms.Form):
    email = forms.EmailField()

    def send(self):
        url = "{0}/users/{1}/password".format(settings.TSURU_HOST,
                                              self.cleaned_data['email'])
        requests.post(url)


class LoginForm(forms.Form):
    username = forms.EmailField(max_length=60, widget=forms.TextInput(attrs={'placeholder': 'Username'}))
    password = forms.CharField(widget=widgets.PasswordInput(attrs={'placeholder': 'Password'}), min_length=6)


class AddUserToTeamForm(forms.Form):

    def __init__(self, teams=None, *args, **kwargs):
        super(AddUserToTeamForm, self).__init__(*args, **kwargs)
        if teams:
            choices = []
            for team in teams:
                choices.append((team, team))
                self.fields["team"].choices = choices

    email = forms.EmailField(max_length=60)
    team = forms.ChoiceField(choices=[])


class SignupForm(forms.Form):
    email = forms.EmailField(max_length=60)
    password = forms.CharField(widget=widgets.PasswordInput, min_length=6)
    same_password_again = forms.CharField(widget=widgets.PasswordInput,
                                          min_length=6)

    def clean(self):
        cleaned_data = super(SignupForm, self).clean()
        password = cleaned_data.get("password")
        same_password_again = cleaned_data.get("same_password_again")

        if not password == same_password_again:
            msg = "You must type the same password twice!"
            self._errors["same_password_again"] = self.error_class([msg])
            raise forms.ValidationError(msg)

        return cleaned_data


class KeyForm(forms.Form):
    name = forms.CharField()
    key = forms.CharField(widget=forms.Textarea)



from dateutil import parser

import mock
import httpretty
import json

from django.test import TestCase
from django.test.client import RequestFactory

from tsuru_dashboard import settings
from tsuru_dashboard.admin.views import PoolInfo


class PoolInfoViewTest(TestCase):
    def setUp(self):
        self.maxDiff = None
        self.request = RequestFactory().get("/")
        self.request.session = {"tsuru_token": "admin"}

    @httpretty.activate
    @mock.patch("tsuru_dashboard.auth.views.token_is_valid")
    def test_should_use_list_template(self, token_is_valid):
        token_is_valid.return_value = True

        url = "{}/docker/node".format(settings.TSURU_HOST)
        body = json.dumps({})
        httpretty.register_uri(httpretty.GET, url, body=body, status=200)

        response = PoolInfo.as_view()(self.request, pool="mypool")

        self.assertIn("admin/pool_info.html", response.template_name)
        self.assertEqual([], response.context_data["nodes"])

    @httpretty.activate
    @mock.patch("tsuru_dashboard.auth.views.token_is_valid")
    def test_should_pass_addresses_to_the_template(self, token_is_valid):
        token_is_valid.return_value = True

        data = {
            "machines": None,
            "nodes": [
                {"Address": "http://128.0.0.1:4243",
                    "Metadata": {"LastSuccess": "2014-08-01T14:09:40-03:00",
                                 "pool": "mypool"},
                 "Status": "ready"},
                {"Address": "http://127.0.0.1:2375",
                 "Metadata": {"LastSuccess": "2014-08-01T14:09:40-03:00",
                              "pool": "mypool"},
                 "Status": "ready"},
                {"Address": "http://myserver.com:2375",
                 "Metadata": {"LastSuccess": "2014-08-01T14:09:40-03:00",
                              "pool": "theonepool"},
                 "Status": "ready"},
                {"Address": "http://myserver2.com:2375",
                 "Metadata": {"LastSuccess": "2014-08-01T14:09:40-03:00"},
                 "Pool": "mypool",
                 "Status": "ready"},
            ],
        }

        url = "{}/docker/node".format(settings.TSURU_HOST)
        body = json.dumps(data)
        httpretty.register_uri(httpretty.GET, url, body=body, status=200)

        for addr in ["http://128.0.0.1:4243", "http://myserver.com:2375", "http://127.0.0.1:2375", "http://myserver2.com:2375"]:
            url = "{}/docker/node/{}/containers".format(settings.TSURU_HOST, addr)
            body = json.dumps(
                [{"Status": "started", "HostAddr": addr, "IP": addr}, {"Status": "stopped", "HostAddr": addr, "IP": addr}])
            httpretty.register_uri(httpretty.GET, url, body=body, status=200)

        response = PoolInfo.as_view()(self.request, pool="mypool")
        date = parser.parse("2014-08-01T14:09:40-03:00")
        expected = [
            {"address": "http://128.0.0.1:4243",
             "units_stats": {"started": 1, "stopped": 1, "total": 2},
             "metadata": {"LastSuccess": "2014-08-01T14:09:40-03:00", "pool": "mypool"},
             "pool": "mypool",
             "last_success": date,
             "units": [{"HostAddr": "http://128.0.0.1:4243", "IP": "http://128.0.0.1:4243", "Status": "started"},
                       {"HostAddr": "http://128.0.0.1:4243", "IP": "http://128.0.0.1:4243", "Status": "stopped"}],
             "status": "ready"},
            {"address": "http://127.0.0.1:2375",
             "units_stats": {"started": 1, "stopped": 1, "total": 2},
             "metadata": {"LastSuccess": "2014-08-01T14:09:40-03:00", "pool": "mypool"},
             "pool": "mypool",
             "units": [{"HostAddr": "http://127.0.0.1:2375", "IP": "http://127.0.0.1:2375", "Status": "started"},
                       {"HostAddr": "http://127.0.0.1:2375", "IP": "http://127.0.0.1:2375", "Status": "stopped"}],
             "last_success": date,
             "status": "ready"},
            {"address": "http://myserver2.com:2375",
             "units_stats": {"started": 1, "stopped": 1, "total": 2},
             "metadata": {"LastSuccess": "2014-08-01T14:09:40-03:00"},
             "pool": "mypool",
             "units": [{"HostAddr": "http://myserver2.com:2375", "IP": "http://myserver2.com:2375", "Status": "started"},
                       {"HostAddr": "http://myserver2.com:2375", "IP": "http://myserver2.com:2375", "Status": "stopped"}],
             "last_success": date,
             "status": "ready"},
        ]
        self.assertEqual(expected, response.context_data["nodes"])

    @httpretty.activate
    @mock.patch("tsuru_dashboard.auth.views.token_is_valid")
    def test_without_units_by_node(self, token_is_valid):
        token_is_valid.return_value = True

        data = {
            "machines": None,
            "nodes": [
                {"Address": "http://128.0.0.1:4243",
                    "Metadata": {"LastSuccess": "2014-08-01T14:09:40-03:00",
                                 "pool": "mypool"},
                 "Status": "ready"},
                {"Address": "http://127.0.0.1:2375",
                 "Metadata": {"LastSuccess": "2014-08-01T14:09:40-03:00",
                              "pool": "mypool"},
                 "Status": "ready"},
                {"Address": "http://myserver.com:2375",
                 "Metadata": {"LastSuccess": "2014-08-01T14:09:40-03:00",
                              "pool": "theonepool"},
                 "Status": "ready"},
            ],
        }

        url = "{}/docker/node".format(settings.TSURU_HOST)
        body = json.dumps(data)
        httpretty.register_uri(httpretty.GET, url, body=body, status=200)

        for addr in ["http://128.0.0.1:4243", "http://myserver.com:2375", "http://127.0.0.1:2375"]:
            url = "{}/docker/node/{}/containers".format(settings.TSURU_HOST, addr)
            body = json.dumps([{"Status": "started"}, {"Status": "stopped"}])
            httpretty.register_uri(httpretty.GET, url, body=body, status=403)

        response = PoolInfo.as_view()(self.request, pool="mypool")
        date = parser.parse("2014-08-01T14:09:40-03:00")
        expected = [
            {"address": "http://128.0.0.1:4243",
             "units_stats": {},
             "units": [],
             "pool": "mypool",
             "metadata": {"LastSuccess": "2014-08-01T14:09:40-03:00", "pool": "mypool"},
             "last_success": date,
             "status": "ready"},
            {"address": "http://127.0.0.1:2375",
             "units_stats": {},
             "units": [],
             "pool": "mypool",
             "metadata": {"LastSuccess": "2014-08-01T14:09:40-03:00", "pool": "mypool"},
             "last_success": date,
             "status": "ready"},
        ]
        self.assertEqual(expected, response.context_data["nodes"])



import os
import json


TSURU_HOST = os.environ.get("TSURU_HOST", "http://localhost:8080")
ELASTICSEARCH_HOST = os.environ.get("ELASTICSEARCH_HOST")
PROMETHEUS_HOST = os.environ.get("PROMETHEUS_HOST")
ELASTICSEARCH_INDEX = os.environ.get("ELASTICSEARCH_INDEX", ".measures-tsuru")
METRICS_COMPONENTS = os.environ.get("METRICS_COMPONENTS", "registry, big-sibling")
RESOLVE_CONNECTION_HOSTS = os.environ.get("RESOLVE_CONNECTION_HOSTS", "") in ['true', 'True', '1']
ELASTICSEARCH_METRICS_ENABLED = os.environ.get("ELASTICSEARCH_METRICS_ENABLED", "true") in ['true', 'True', '1']

GRAFANA_DASHBOARD = os.environ.get("GRAFANA_DASHBOARD")
GRAFANA_POOL_DASHBOARD = os.environ.get("GRAFANA_POOL_DASHBOARD")
GRAFANA_THEME = os.environ.get("GRAFANA_THEME", "light")
GRAFANA_KIOSK = os.environ.get("GRAFANA_KIOSK", "tv")
GRAFANA_DEFAULT_DATASOURCE = os.environ.get("GRAFANA_DEFAULT_DATASOURCE")
GRAFANA_DATASOURCE_FOR_POOL = json.loads(os.environ.get("GRAFANA_DATASOURCE_FOR_POOL", "{}"))



from mock import patch, Mock

from django.test import TestCase
from django.test.client import RequestFactory

from tsuru_dashboard import settings
from tsuru_dashboard.apps.views import ListAppJson

import json


class ListAppJsonViewTest(TestCase):
    def setUp(self):
        self.request = RequestFactory().get("/")
        self.request.session = {"tsuru_token": "admin"}

    @patch('requests.get')
    def teste_list_apps(self, get):
        expected = [{
            "framework": "python",
            "name": "pacote",
            "repository": "git@tsuru.com:pacote.git",
            "state": "creating"
        }]
        response_mock = Mock(status_code=200)
        response_mock.json.return_value = expected
        get.return_value = response_mock

        response = ListAppJson.as_view()(self.request)

        expected = {
            "apps": [{
                "framework": "python",
                "name": "pacote",
                "repository": "git@tsuru.com:pacote.git",
                "state": "creating",
            }]
        }
        self.assertDictEqual(expected, json.loads(response.content))
        url = "{}/apps?simplified=true".format(settings.TSURU_HOST)
        get.assert_called_with(url, headers={'authorization': 'admin'})

    @patch('requests.get')
    def teste_list_apps_by_name(self, get):
        expected = [{
            "framework": "python",
            "name": "pacote",
            "repository": "git@tsuru.com:pacote.git",
            "state": "creating"
        }]
        response_mock = Mock(status_code=200)
        response_mock.json.return_value = expected
        get.return_value = response_mock

        request = RequestFactory().get("/?name=pacote")
        request.session = {"tsuru_token": "admin"}
        response = ListAppJson.as_view()(request)

        expected = {
            "apps": [{
                "framework": "python",
                "name": "pacote",
                "repository": "git@tsuru.com:pacote.git",
                "state": "creating",
            }]
        }
        self.assertDictEqual(expected, json.loads(response.content))
        url = "{}/apps?simplified=true&name=pacote".format(settings.TSURU_HOST)
        get.assert_called_with(url, headers={'authorization': 'admin'})



from django.forms import EmailField
from django.test import TestCase

from tsuru_dashboard.auth import forms


class AddUserToTeamFormTest(TestCase):

    def test_should_have_email_and_team_fields(self):
        self.assertIn('email', forms.AddUserToTeamForm.base_fields)
        self.assertIn('team', forms.AddUserToTeamForm.base_fields)

    def test_email_should_have_at_most_60_characters(self):
        field = forms.AddUserToTeamForm.base_fields['email']
        self.assertEqual(60, field.max_length)

    def test_email_field_should_accept_only_email_values(self):
        field = forms.AddUserToTeamForm.base_fields['email']
        self.assertIsInstance(field, EmailField)

    def test_should_receive_list_of_choices_via_constructor(self):
        form = forms.AddUserToTeamForm(teams=["team1", "team2"])
        expected = [("team1", "team1"), ("team2", "team2")]
        self.assertEqual(expected, form.fields["team"].choices)
        self.assertEqual(expected, form.fields["team"].widget.choices)

    def test_choices_should_be_empty_when_no_team_list_is_given(self):
        form = forms.AddUserToTeamForm()
        self.assertEqual([], form.fields["team"].choices)



apps = {}


class AppNotFound(Exception):
    pass


class ObjectIsNotApp(Exception):
    pass


class TabNotFound(Exception):
    pass


class Tab(object):
    pass


class App(object):
    def __init__(self):
        self.tabs = []

    def register_tab(self, tab):
        self.tabs.append(tab)

    def get_tab(self, tab_name):
        for tab in self.tabs:
            if tab_name == tab.name:
                return tab

        raise TabNotFound

    def unregister_tab(self, tab_name):
        tab = self.get_tab(tab_name)
        self.tabs.remove(tab)


def register(app):
    if not issubclass(app, App):
        raise ObjectIsNotApp

    apps[app.name] = app()


def get(app_name):
    if app_name not in apps:
        raise AppNotFound

    return apps[app_name]


def unregister(app_name):
    del apps[app_name]



from django.test import TestCase
from django.test.client import RequestFactory
from django.core.urlresolvers import reverse

from tsuru_dashboard import settings
from tsuru_dashboard.teams.views import Remove

import mock


class RemoveViewTest(TestCase):
    def setUp(self):
        self.request = RequestFactory().get("/")
        self.request.session = {"tsuru_token": "admin"}

    @mock.patch("django.contrib.messages.error")
    @mock.patch("requests.delete")
    @mock.patch("requests.get")
    def test_view(self, get, delete, error):
        get.return_value = mock.Mock(status_code=200)
        team_name = "avengers"
        response = Remove.as_view()(self.request, team=team_name)
        self.assertEqual(302, response.status_code)
        self.assertEqual(reverse("team-list"), response.items()[2][1])
        delete.assert_called_with(
            '{0}/teams/{1}'.format(
                settings.TSURU_HOST,
                team_name),
            headers={'authorization': 'admin'})

    @mock.patch("django.contrib.messages.error")
    @mock.patch("requests.delete")
    @mock.patch("requests.get")
    def test_view_should_send_error_message(self, get, delete, error):
        get.return_value = mock.Mock(status_code=200)
        delete.return_value = mock.Mock(status_code=403, text=u'Can not delete this team!')
        team_name = "avengers"
        Remove.as_view()(self.request, team=team_name)
        error.assert_called_with(self.request, u'Can not delete this team!', fail_silently=True)



from mock import patch, Mock

from django.test import TestCase
from django.test.client import RequestFactory

from tsuru_dashboard.admin.views import NodeAdd

import json


class FakeTsuruClient(object):
    def __init__(self, error=""):
        create = Mock()
        attrs = {"iter_lines.return_value": [json.dumps({"Message": "", "Error": error})]}
        create.configure_mock(**attrs)

        self.nodes = Mock()
        attrs = {"create.return_value": create}
        self.nodes.configure_mock(**attrs)


class NodeAddViewTest(TestCase):
    @patch.object(NodeAdd, "client", FakeTsuruClient(error="something is wrong"))
    @patch("tsuru_dashboard.auth.views.token_is_valid")
    def test_view_register_false(self, token_is_valid):
        token_is_valid.return_value = True

        request = RequestFactory().post('/', data={"key": "value", "register": "true"})
        request.session = {'tsuru_token': 'tokentest'}
        response = NodeAdd.as_view()(request)

        self.assertEqual(response.status_code, 500)
        self.assertEqual(response.content, "something is wrong")

    @patch.object(NodeAdd, "client", FakeTsuruClient())
    @patch("tsuru_dashboard.auth.views.token_is_valid")
    def test_view_register_true(self, token_is_valid):
        token_is_valid.return_value = True

        request = RequestFactory().post('/', data={"key": "value", "register": "true"})
        request.session = {'tsuru_token': 'tokentest'}

        response = NodeAdd.as_view()(request)

        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.content, "Node was successfully created")



from mock import patch, Mock

from django.test import TestCase
from django.test.client import RequestFactory
from django.core.urlresolvers import reverse

from tsuru_dashboard import settings
from tsuru_dashboard.auth.views import LoginRequiredMixin, KeysAdd
from tsuru_dashboard.auth.forms import KeyForm


class KeysAddViewTest(TestCase):
    @patch("requests.get")
    def setUp(self, get):
        get.return_value = Mock(status_code=200)
        self.factory = RequestFactory()
        self.request = self.factory.get('/')
        self.request.session = {"tsuru_token": "admin"}
        self.response = KeysAdd.as_view()(self.request)
        self.request_post = self.factory.post('/add-keys/', {'key': 'test-key-qq', 'name': 'mykey'})
        self.request_post.session = {"tsuru_token": "admin"}

    def test_should_require_login_to_create_team(self):
        assert issubclass(KeysAdd, LoginRequiredMixin)

    def test_key_should_render_expected_template(self):
        self.assertIn('auth/key_add.html', self.response.template_name)

    def test_context_should_contain_form(self):
        self.assertIn('form', self.response.context_data.keys())

    def test_form_in_context_should_has_a_instance_of_KeyForm(self):
        form = self.response.context_data.get('form')
        self.assertIsInstance(form, KeyForm)

    def test_get_request_key_url_should_not_return_404(self):
        response = self.client.get(reverse('add-keys'))
        self.assertNotEqual(404, response.status_code)

    @patch("django.contrib.messages.error")
    @patch('requests.post')
    @patch('requests.get')
    def test_post_with_name_should_send_request_post_to_tsuru(self, get, post, er):
        get.return_value = Mock(status_code=200)
        self.request_post.session = {'tsuru_token': 'tokentest'}
        KeysAdd.as_view()(self.request_post)
        self.assertEqual(1, post.call_count)
        post.assert_called_with(
            '%s/users/keys' % settings.TSURU_HOST,
            data={"name": "mykey", "key": "test-key-qq"},
            headers={'authorization':
                     self.request_post.session['tsuru_token']})

    @patch("django.contrib.messages.success")
    @patch('requests.post')
    @patch('requests.get')
    def test_valid_postshould_return_message_expected(self, get, post, success):
        get.return_value = Mock(status_code=200)
        post.return_value = Mock(status_code=200)
        KeysAdd.as_view()(self.request_post)
        success.assert_called_with(self.request_post, "The key was successfully added", fail_silently=True)

    @patch("django.contrib.messages.error")
    @patch('requests.post')
    @patch('requests.get')
    def test_invalid_post_should_return_error_message(self, get, post, error):
        get.return_value = Mock(status_code=200)
        post.return_value = Mock(status_code=500, text='Error')
        KeysAdd.as_view()(self.request_post)
        error.assert_called_with(self.request_post, 'Error', fail_silently=True)

    @patch("django.contrib.messages.success")
    @patch('requests.post')
    @patch('requests.get')
    def test_successfully_post_should_redirects(self, get, post, m):
        get.return_value = Mock(status_code=200)
        post.return_value = Mock(status_code=200)
        response = KeysAdd.as_view()(self.request_post)
        self.assertEqual(302, response.status_code)
        self.assertEqual(reverse('add-keys'), response.items()[2][1])

    @patch("django.contrib.messages.error")
    @patch('requests.post')
    @patch('requests.get')
    def test_post_with_error_should_redirects(self, get, post, er):
        get.return_value = Mock(status_code=200)
        post.return_value = Mock(status_code=500, content='Error')
        response = KeysAdd.as_view()(self.request_post)
        self.assertEqual(302, response.status_code)
        self.assertEqual(reverse('add-keys'), response.items()[2][1])

    @patch('requests.post')
    @patch('requests.get')
    def test_post_without_key_should_not_send_request_to_tsuru(self, get, post):
        get.return_value = Mock(status_code=200)
        request = self.factory.post('/team/', {'key': ''})
        request.session = {}
        KeysAdd.as_view()(request)
        self.assertEqual(0, post.call_count)

    @patch('requests.post')
    @patch('requests.get')
    def test_post_without_key_should_return_form_with_errors(self, get, post):
        get.return_value = Mock(status_code=200)
        request = self.factory.post('/team/', {'key': ''})
        request.session = {"tsuru_token": "admin"}
        response = KeysAdd.as_view()(request)
        self.assertIn('form', response.context_data.keys())
        form = response.context_data.get('form')
        self.assertIsInstance(form, KeyForm)
        self.assertEqual(u'This field is required.',
                         form.errors.get('key')[0])



from django import template
from django.urls import reverse


register = template.Library()


@register.simple_tag
def event_url(event_id, app_name=''):
    if app_name != '':
        return reverse('app-event', args=[app_name, event_id])
    return reverse('event-info', args=[event_id])



import json

from mock import patch, Mock

from django.test import TestCase
from django.test.client import RequestFactory

from tsuru_dashboard.apps.views import CreateApp
from tsuru_dashboard.apps import forms
from tsuruclient.base import TsuruAPIError


class FakeTsuruClient(object):
    def __init__(self, exception=False, return_value=""):
        attrs = {}
        if exception:
            attrs["create.side_effect"] = TsuruAPIError(return_value)
        else:
            attrs["create.return_value"] = return_value
        self.apps = Mock(**attrs)


class CreateAppViewTest(TestCase):
    @patch("tsuru_dashboard.auth.views.token_is_valid")
    def setUp(self, token_is_valid):
        token_is_valid.return_value = True
        self.request = RequestFactory().get("/")
        self.request.session = {'tsuru_token': 'tokentest'}

    @patch('requests.get')
    def test_should_use_create_template(self, get):
        content = u"""[{"Name":"python"},{"Name":"ruby"},{"Name":"static"}]"""
        m = Mock(status_code=200, content=content)
        m.json.return_value = json.loads(content)
        get.return_value = m
        request = RequestFactory().get("/")
        request.session = {}

        view = CreateApp()
        view.plans = lambda r: ("small", [("small", "small")])
        view.teams = lambda r: []
        view.pools = lambda r: []
        response = view.get(request)

        self.assertEqual("apps/create.html", response.template_name)

    @patch('requests.get')
    def test_AppForm_should_be_in_context(self, get):
        content = u"""[{"Name":"python"},{"Name":"ruby"},{"Name":"static"}]"""
        m = Mock(status_code=200, content=content)
        m.json.return_value = json.loads(content)
        get.return_value = m
        request = RequestFactory().get("/")
        request.session = {}

        view = CreateApp()
        view.plans = lambda r: ("basic", [("basic", "basic")])
        view.teams = lambda r: []
        view.pools = lambda r: []
        response = view.get(request)

        app_form = response.context_data['app_form']
        self.assertIsInstance(app_form, forms.AppForm)

    @patch.object(CreateApp, "client", FakeTsuruClient(exception=True, return_value="Error creating app"))
    @patch("django.contrib.messages.error")
    @patch('requests.get')
    def test_post_with_errors_should_show_error_message(self, get, error):
        content = u"""[{"Name":"python"},{"Name":"ruby"},{"Name":"static"}]"""
        m = Mock(status_code=200, content=content)
        m.json.return_value = json.loads(content)
        get.return_value = m
        request = RequestFactory().post(
            "/",
            {"name": "myepe", "platform": "python"})
        request.session = {}

        view = CreateApp()
        view.plans = lambda r: ("small", [("small", "small")])
        view.teams = lambda r: []
        view.pools = lambda r: []

        view.post(request)

        error.assert_called_with(request, u'Error creating app', fail_silently=True)

    @patch('requests.get')
    def test_post_without_name_should_return_form_with_errors(self, get):
        content = u"""[{"Name":"python"},{"Name":"ruby"},{"Name":"static"}]"""
        m = Mock(status_code=200, content=content)
        m.json.return_value = json.loads(content)
        get.return_value = m
        request = RequestFactory().post("/", {"name": ""})
        request.session = {}

        view = CreateApp()
        view.plans = lambda r: ("small", [("small", "small")])
        view.teams = lambda r: []
        view.pools = lambda r: []
        response = view.post(request)

        form = response.context_data.get('app_form')
        self.assertIn('name', form.errors)
        self.assertIn(u'This field is required.', form.errors.get('name'))

    @patch.object(CreateApp, "client", FakeTsuruClient())
    @patch('requests.get')
    def test_post_should_send_to_tsuru_removing_empty_keys(self, get):
        data = {"name": "myepe", "platform": "django", "plan": ""}
        request = RequestFactory().post("/", data)
        request.session = {'tsuru_token': 'tokentest'}

        view = CreateApp()
        view.plans = lambda r: ("basic", [("basic", "basic")])
        view.platforms = lambda r: [("django", "django")]
        view.post(request)

        CreateApp.client.apps.create.assert_called_with(name="myepe", platform="django")

    @patch.object(CreateApp, "client", FakeTsuruClient())
    @patch('requests.get')
    def test_post_should_split_tags(self, get):
        data = {"name": "myepe", "platform": "django", "tags": " tag 1 , tag 2, tag 3  ,, , "}
        request = RequestFactory().post("/", data)
        request.session = {'tsuru_token': 'tokentest'}

        view = CreateApp()
        view.plans = lambda r: ("basic", [("basic", "basic")])
        view.platforms = lambda r: [("django", "django")]
        view.post(request)

        CreateApp.client.apps.create.assert_called_with(name="myepe", platform="django", tag=["tag 1", "tag 2", "tag 3"])

    @patch.object(CreateApp, "client", FakeTsuruClient())
    @patch("django.contrib.messages.success")
    @patch('requests.get')
    def test_post_with_success_should_redirect_to_app_list(self, get, success):
        data = {"name": "myepe", "platform": "django", "plan": ""}
        request = RequestFactory().post("/", data)
        request.session = {'tsuru_token': 'tokentest'}

        view = CreateApp()
        view.plans = lambda r: ("basic", [("basic", "basic")])
        view.platforms = lambda r: [("django", "django")]
        response = view.post(request)

        success.assert_called_with(request, u'App was successfully created', fail_silently=True)
        self.assertEquals(302, response.status_code)
        self.assertEquals("/apps/", response.url)

    @patch('requests.get')
    def test_pools_when_api_returns_pools_in_older_format(self, get):
        mocked_response = Mock(status_code=200)
        mocked_response.json.return_value = json.loads('[{"Name": "dead"}]')
        get.return_value = mocked_response
        view = CreateApp()
        pools = view.pools(self.request)
        self.assertListEqual([("", ""), ("dead", "dead")], pools)

    @patch("requests.get")
    def test_pools_when_api_returns_no_content(self, get):
        mocked_response = Mock(status_code=204)
        get.return_value = mocked_response
        view = CreateApp()
        pools = view.pools(self.request)
        self.assertListEqual([('', '')], pools)

    @patch('requests.get')
    def test_pools_when_api_returns_pools_in_new_format(self, get):
        mocked_response = Mock(status_code=200)
        mocked_response.json.return_value = json.loads('[{"name": "theonepool"}]')
        get.return_value = mocked_response
        view = CreateApp()
        pools = view.pools(self.request)
        self.assertEqual([("", ""), ("theonepool", "theonepool")], pools)

    @patch('requests.get')
    def test_teams(self, get):
        content = u"""[{"name": "team1"}]"""
        m = Mock(status_code=200, content=content)
        m.json.return_value = json.loads(content)
        get.return_value = m
        request = RequestFactory().get("/")
        request.session = {"tsuru_token": "tokentest"}

        view = CreateApp()
        teams = view.teams(request)
        self.assertEqual([("", ""), ('team1', 'team1')], teams)

    @patch('requests.get')
    def test_teams_empty(self, get):
        m = Mock(status_code=204)
        get.return_value = m
        request = RequestFactory().get("/")
        request.session = {"tsuru_token": "tokentest"}

        view = CreateApp()
        teams = view.teams(request)
        self.assertEqual([("", "")], teams)

    @patch('requests.get')
    def test_plans(self, get):
        content = u"""[{"name": "plan1"}]"""
        m = Mock(status_code=200, content=content)
        m.json.return_value = json.loads(content)
        get.return_value = m
        request = RequestFactory().get("/")
        request.session = {"tsuru_token": "tokentest"}

        view = CreateApp()
        plans = view.plans(request)
        self.assertEqual(("", [("", ""), ('plan1', 'plan1')]), plans)

    @patch('requests.get')
    def test_plans_is_None(self, get_mock):
        response_mock = Mock()
        response_mock.json.return_value = None
        get_mock.return_value = response_mock

        data = {"name": "myepe", "platform": "django", "plan": "basic"}
        request = RequestFactory().post("/", data)
        request.session = {'tsuru_token': 'tokentest'}

        view = CreateApp()
        default, plans = view.plans(request)
        self.assertEqual(default, '')
        self.assertListEqual([('', '')], plans)



from dateutil import parser
from pytz import utc
from urlparse import urlparse
from collections import defaultdict


def extract_ip(address):
    if not urlparse(address).scheme:
        address = "http://"+address
    return urlparse(address).hostname


class Node():
    def __init__(self, data, units_response=None):
        self.data = data or {}
        self.load_units(units_response)

    def load_units(self, response):
        self.units = []
        if not response or response.status_code != 200:
            return

        node_units = response.json()
        if not node_units:
            return

        addr = node_units[0].get('IP') or node_units[0].get('Ip')
        if not addr:
            return

        if extract_ip(addr) == extract_ip(self.address()):
            self.units = node_units

    def address(self):
        return self.data.get('Address')

    def last_success(self):
        date = self.metadata().get("LastSuccess")
        if date:
            last_success = parser.parse(date)
            if last_success.tzinfo:
                last_success = last_success.astimezone(utc)
            else:
                last_success = utc.localize(last_success)
            return last_success
        return date

    def metadata(self):
        return self.data.get("Metadata") or {}

    def pool(self):
        pool = self.data.get("Pool", self.data.get("pool"))
        if pool:
            return pool
        return str(self.metadata().get("pool", ""))

    def status(self):
        return self.data.get("Status")

    def units_stats(self):
        result = defaultdict(int)
        for unit in self.units:
            unit_status = unit.get('Status') or unit.get('status')
            if unit_status:
                result[unit_status] += 1

        len_units = len(self.units)
        if len_units > 0:
            result['total'] = len_units

        return result

    def to_dict(self):
        return {
            'address': self.address(),
            'metadata': self.metadata(),
            'last_success': self.last_success(),
            'pool': self.pool(),
            'status': self.status(),
            'units_stats': self.units_stats(),
            'units': self.units,
        }



from django.test import TestCase

from tsuru_dashboard.apps import forms


class AppFormTest(TestCase):

    def test_forms_should_have_AppForm(self):
        self.assertTrue(hasattr(forms, "AppForm"))

    def test_fields(self):
        fields = ["name", "platform", "teamOwner", "pool", "plan", "tags"]
        for field_name in fields:
            self.assertIn(field_name, forms.AppForm.base_fields)

    def test_required_false_fields(self):
        fields = ["teamOwner", "pool", "plan", "tags"]
        for field_name in fields:
            self.assertFalse(forms.AppForm.base_fields[field_name].required)

    def test_required_fields(self):
        fields = ["name", "platform"]
        for field_name in fields:
            self.assertTrue(forms.AppForm.base_fields[field_name].required)



from django.test import TestCase

from . import register, get, unregister, AppNotFound, App, TabNotFound


class RegisterTest(TestCase):
    def test_register(self):
        class MyApp(App):
            name = 'myapp'

        register(MyApp)
        my_app = get('myapp')

        self.assertIsInstance(my_app, MyApp)

    def test_unregister(self):
        class MyApp(App):
            name = 'myapp'

        register(MyApp)
        unregister('myapp')

        with self.assertRaises(AppNotFound):
            get('myapp')


class AppTest(TestCase):
    def test_register_tab(self):
        class MyTab(object):
            name = 'mytab'

        class MyApp(App):
            name = 'myapp'

        my_app = MyApp()
        my_app.register_tab(MyTab)
        my_tab = my_app.get_tab('mytab')

        self.assertEqual(my_tab, MyTab)

    def test_unregister_tab(self):
        class MyTab(object):
            name = 'mytab'

        class MyApp(App):
            name = 'myapp'

        my_app = MyApp()
        my_app.register_tab(MyTab)

        my_app.unregister_tab('mytab')

        with self.assertRaises(TabNotFound):
            my_app.get_tab('mytab')



from django.test import TestCase
from django.urls import resolve


class TestAdminUrls(TestCase):

    def test_urls(self):
        tests = [
            ["/", "pool-list"],
            ["/192.168.1.1/containers/", "node-info-json"],
            ["/192.168.1.1/", "node-info"],
            ["/deploys/abc123/", "deploy-info"],
            ["/deploys/", "list-deploys"],
            ["/healing/", "list-healing"],
            ["/node/192.168.1.1:8785/remove/", "node-remove"],
            ["/node/add/", "node-add"],
            ["/pool/mypool/rebalance/", "pool-rebalance"],
            ["/pool/mypool/", "pool-info"],
            ["/templates.json", "template-list-json"],
        ]

        for test in tests:
            resolver = resolve("/admin" + test[0])
            self.assertEqual(resolver.view_name, test[1])



from django.test import TestCase
from django.test.client import RequestFactory
from django.http import HttpResponse

from tsuru_dashboard.apps.views import RemoveApp

import mock


class RemoveAppTestCase(TestCase):
    def setUp(self):
        self.request = RequestFactory().delete("/name/remove")
        self.request.session = {"tsuru_token": "admin"}

    @mock.patch('requests.delete')
    @mock.patch('requests.get')
    def test_should_returns_404_when_app_does_not_exists(self, get, delete):
        get.return_value = mock.Mock(status_code=200)
        delete.return_value = mock.Mock(status_code=404, text="app not found")
        response = RemoveApp.as_view()(self.request, name="appname")
        self.assertEqual(404, response.status_code)
        self.assertEqual("app not found", response.content)

    @mock.patch('requests.delete')
    @mock.patch('requests.get')
    def test_should_redirect_to_the_app_list(self, get, delete):
        get.return_value = mock.Mock(status_code=200)
        delete.return_value = mock.Mock(status_code=200)
        response = RemoveApp.as_view()(self.request, name="appname")
        self.assertEqual(200, response.status_code)
        self.assertIsInstance(response, HttpResponse)
